export function recordToMap<T>(r: Record<string, T>): Map<string, T> {
	const ret = new Map();

	for (const k of Object.keys(r))
		ret.set(k, r[k]);

	return ret;
}

export function mapValues<K, V1, V2>(f: (v: V1) => V2, map: Map<K, V1>): Map<K, V2> {
	const ret: Map<K, V2> = new Map();

	for (const [k, v] of map)
		ret.set(k, f(v));

	return ret;
}

export async function mapValuesAsync<K, V1, V2>(f: (v: V1) => Promise<V2>, map: Map<K, V1>): Promise<Map<K, V2>> {
	const ret: Map<K, V2> = new Map();

	for (const [k, v] of mapValues(f, map))
		ret.set(k, await v);

	return ret;
}

export function nextPowerOf2(n: number): number {
	let i = 1;

	while (i < n)
		i *= 2;

	return i;
}

export class Listenable<T extends any[]> {
	private readonly listeners: Array<(...args: T) => void> = [];

	public addListener(listener: (...args: T) => void): void {
		this.listeners.push(listener);
	}

	protected runListeners(...args: T): void {
		this.listeners.forEach((l) => l(...args));
	}
}

export async function getJSON(url: string): Promise<any> {
	const res = await window.fetch(url);
	return await res.json();
}

export function nextAnimationFrame(): Promise<DOMHighResTimeStamp> {
	return new Promise((resolve) => window.requestAnimationFrame(resolve));
}
